package net.wanji.utc.controller;

import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import net.wanji.utc.entity.JsonViewObject;
import net.wanji.utc.po.ManufacturerApiInfoPO;
import net.wanji.utc.service.systemadmin.ManufacturerApiInfoService;
import net.wanji.utc.vo.systemadmin.DeleteByIntegerIdListInVO;
import net.wanji.utc.vo.systemadmin.ManufacturerApiInfoInsertOrUpdateInVO;
import net.wanji.utc.vo.systemadmin.ManufacturerApiInfoListInVO;
import net.wanji.utc.vo.systemadmin.ManufacturerApiInfoOutVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.ws.rs.core.MediaType;

/**
 * 系统管理-接口管理
 *
 * @author Kent HAN
 * @date 2022/11/24 8:32
 */

@Api(value = "系统管理-接口管理", description = "系统管理-接口管理")
@RequestMapping("/manufacturerApiInfo")
@RestController
public class ManufacturerApiController {
    @Autowired
    ManufacturerApiInfoService manufacturerApiInfoService;

    @ApiOperation(value = "接口列表", notes = "接口列表", response = ManufacturerApiInfoPO.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/list",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = ManufacturerApiInfoPO.class),
    })
    public JsonViewObject list(@RequestBody ManufacturerApiInfoListInVO manufacturerApiInfoListInVO) {
        PageInfo<ManufacturerApiInfoOutVO> manufacturerApiInfoOutVOPageInfo =
                manufacturerApiInfoService.list(manufacturerApiInfoListInVO);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(manufacturerApiInfoOutVOPageInfo);
    }

    @ApiOperation(value = "接口添加或修改", notes = "接口添加或修改", consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/insertOrUpdate", consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject insertOrUpdate(@RequestBody ManufacturerApiInfoInsertOrUpdateInVO inVO) {
        manufacturerApiInfoService.insertOrUpdate(inVO);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success();
    }

    @ApiOperation(value = "接口删除", notes = "接口删除", consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/delete", consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject delete(@RequestBody DeleteByIntegerIdListInVO inVO) {
        manufacturerApiInfoService.delete(inVO);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success();
    }
}
