package net.wanji.utc.controller;

import net.wanji.utc.common.constant.Constants;
import net.wanji.utc.service.staticinfo.StaticInfoService;
import net.wanji.utc.common.baseentity.BaseCrossInfo;
import net.wanji.utc.vo.CrossInfoInVO;
import net.wanji.utc.common.genericentity.OutVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.wanji.utc.vo.CrossSchedulesInVO;
import net.wanji.utc.vo.PlanSectionInVO;
import net.wanji.utc.vo.SchemePhaseLightsInVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;

/**
 * 静态信息接口
 *
 * @date 2022/11/15 9:38
 */
@Api(value = "静态信息接口", description = "静态信息接口")
@RequestMapping("/staticInfo")
@RestController
public class StaticInfoController {
    @Autowired
    StaticInfoService staticInfoService;

    @PostMapping("/crossInfo")
    @ApiOperation(value = "信号路口基础信息", notes = "信号路口基础信息")
    public ResponseEntity crossInfo(HttpServletRequest request, @RequestBody CrossInfoInVO crossInfoInVO) {
        OutVO<BaseCrossInfo> outVO = new OutVO<>();
        String manufacturerCode = crossInfoInVO.getManufacturerCode();
        outVO.setManufacturerCode(manufacturerCode);
        outVO.setUpdatetime(new Date());
        outVO.setSystemAbbr(Constants.SYSTEM_ABBR);

        staticInfoService.crossBasicInfo(outVO);
        return ResponseEntity.ok("success");

    }

    @PostMapping("/schemePhaseLights")
    @ApiOperation(value = "方案数据-方案信息、相位信息、灯组信息", notes = "方案数据-方案信息、相位信息、灯组信息")
    public ResponseEntity schemePhaseLights(HttpServletRequest request,
                                            @RequestBody SchemePhaseLightsInVO schemePhaseLightsInVO) throws Exception {
        staticInfoService.schemePhaseLights(schemePhaseLightsInVO);
        return ResponseEntity.ok("success");
    }

    @PostMapping("/planSection")
    @ApiOperation(value = "计划数据-计划信息、时段信息", notes = "计划数据-计划信息、时段信息")
    public ResponseEntity planSection(HttpServletRequest request,
                                            @RequestBody PlanSectionInVO planSectionInVO) throws Exception {
        staticInfoService.planSection(planSectionInVO);
        return ResponseEntity.ok("success");
    }

    @PostMapping("/crossSchedules")
    @ApiOperation(value = "时间表数据", notes = "时间表数据")
    public ResponseEntity crossSchedules(HttpServletRequest request,
                                      @RequestBody CrossSchedulesInVO crossSchedulesInVO) throws Exception {
        staticInfoService.crossSchedules(crossSchedulesInVO);
        return ResponseEntity.ok("success");
    }
}
