package net.wanji.utc.handler;

import lombok.extern.slf4j.Slf4j;
import net.wanji.utc.common.Result;
import net.wanji.utc.common.exception.ControlException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.servlet.http.HttpServletRequest;

import static net.wanji.utc.common.ResultEnum.INTERNAL_SERVER_ERROR;

/**
 * @author wuxiaokai
 * @date 2022/11/21 9:06:40
 */
@RestControllerAdvice
@Slf4j
public class GlobalExceptionHandler {

    /**
     * 处理自定义的业务异常
     */
    @ExceptionHandler(value = ControlException.class)
    public Result<String> exceptionHandler(HttpServletRequest req, ControlException e) {
        log.error("发生业务异常！原因是：{}", e.getErrorMsg());
        return Result.error(e.getErrorCode(), e.getErrorMsg());
    }

    /**
     * 处理空指针的异常
     */
    @ExceptionHandler(value = NullPointerException.class)
    public Result<String> exceptionHandler(HttpServletRequest req, NullPointerException e) {
        log.error("发生空指针异常！原因是：{}", e.getMessage());
        return Result.error(INTERNAL_SERVER_ERROR.getResultCode(), e.getMessage());
    }

    /**
     * 处理其他异常
     */
    @ExceptionHandler(value = Exception.class)
    public Result<String> exceptionHandler(HttpServletRequest req, Exception e) {
        log.error("未知异常！原因是：{}", e.getMessage());
        return Result.error(INTERNAL_SERVER_ERROR.getResultCode(), e.getMessage());
    }
}
