package net.wanji.utc.service.control;

import net.wanji.utc.common.Result;
import net.wanji.utc.vo.PhaseLockVO;
import net.wanji.utc.vo.PlanSendVO;
import net.wanji.utc.vo.schedulesend.ScheduleSendVO;
import net.wanji.utc.vo.timeplan.TimePlanVO;

import java.util.List;

/**
 * 控制指令 service
 *
 * @author wuxiaokai
 * @date 2022/11/15 14:02:36
 */
public interface ControlCommandService {

    /**
     * 方案下发
     */
    <T> Result<T> schemeSend();

    /**
     * 计划下发
     *
     * @param planSendVO 计划下发VO
     * @return {@link Result}<{@link T}>
     */
    <T> Result<T> planSend(PlanSendVO planSendVO) throws Exception;

    /**
     * 时间表下发
     *
     * @param scheduleSendVO 时间表下发VO
     * @return {@link Result}<{@link T}>
     */
    <T> Result<T> scheduleSend(ScheduleSendVO scheduleSendVO) throws Exception;

    /**
     * 锁定控制
     *
     * @param phaseLock 相位锁定实体类
     * @return {@link Result}<{@link T}>
     * @throws Exception 异常
     */
    <T> Result<T> lockControl(PhaseLockVO phaseLock) throws Exception;

    /**
     * 步进控制
     *
     * @param code    信号机编号
     * @param command 1 开始步进 0 取消步进
     * @param stepNum 0 顺序步进 n 跳过n个相位
     * @return {@link Result}<{@link T}>
     * @throws Exception 异常
     */
    <T> Result<T> stepControl(String code, Integer command, Integer stepNum) throws Exception;

    /**
     * 设置信号机控制模式
     *
     * @param code        信号机编号
     * @param command     指令 1 开 0 关
     * @param commandType 命令类型
     * @return {@link Result}<{@link T}>
     * @throws Exception 异常
     */
    <T> Result<T> setSignalControl(String code, Integer command, Integer commandType) throws Exception;

    /**
     * 恢复时间表
     *
     * @param code       信号机编号
     * @return {@link Result}<{@link T}>
     */
    <T> Result<T> recoverSchedule(String code) throws Exception;

    /**
     * 相位配时下发
     *
     * @param code      信号机id
     * @param timePlans 批量下发的配时数据
     * @return {@link Result}<{@link T}>
     * @throws InterruptedException 中断异常
     */
    <T> Result<T> phaseTimingSend(String code, List<TimePlanVO> timePlans) throws InterruptedException;

    /**
     * 相位差下发
     */
    <T> Result<T> phaseDiffSend();

}
