package net.wanji.utc.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hikvision.artemis.sdk.ArtemisHttpUtil;
import com.hikvision.artemis.sdk.config.ArtemisConfig;
import lombok.extern.slf4j.Slf4j;
import net.wanji.utc.common.Result;
import net.wanji.utc.common.baseentity.BaseCrossInfo;
import net.wanji.utc.common.commonentity.HttpRequest;
import net.wanji.utc.common.exception.ControlException;
import net.wanji.utc.common.typeenum.BasicEnum;
import net.wanji.utc.entity.SignalRunring;
import net.wanji.utc.mapper.CrossInfoMapper;
import net.wanji.utc.mapper.CrossPhaseMapper;
import net.wanji.utc.po.CrossInfoPO;
import net.wanji.utc.po.CrossPhasePO;
import net.wanji.utc.service.control.ControlCommandService;
import net.wanji.utc.service.runninginfo.HkLightsStatusService;
import net.wanji.utc.util.FieldUtil;
import net.wanji.utc.util.ListUtil;
import net.wanji.utc.vo.LightsStatusVO;
import net.wanji.utc.vo.PhaseLockVO;
import net.wanji.utc.vo.PlanSendVO;
import net.wanji.utc.vo.schedulesend.ScheduleSendVO;
import net.wanji.utc.vo.timeplan.Phase;
import net.wanji.utc.vo.timeplan.Ring;
import net.wanji.utc.vo.timeplan.TimePlanVO;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

import static net.wanji.utc.common.ResultEnum.BODY_NOT_MATCH;
import static net.wanji.utc.common.constant.Constants.*;

/**
 * @author wuxiaokai
 * @date 2022/11/21 9:51:47
 */
@Slf4j
@Service("hkControlCommandService")
public class HKControlCommandServiceImpl implements ControlCommandService {

    @Value("${signal.manufacturer.hk.artemisPath}")
    private String artemisPath;

    @Resource
    private HkLightsStatusService hkLightsStatusService;

    @Resource
    private ArtemisConfig artemisConfig;

    @Resource
    private CrossPhaseMapper crossPhaseMapper;

    @Resource
    private CrossInfoMapper crossInfoMapper;

    @Override
    public <T> Result<T> schemeSend() {
        return null;
    }

    @Override
    public <T> Result<T> planSend(PlanSendVO planSendVO) throws Exception {
        CrossInfoPO crossInfoPo = crossInfoMapper.selectByCode(planSendVO.getCode());
        if (crossInfoPo == null) {
            throw new ControlException(BODY_NOT_MATCH.getResultCode(), "参数错误，信号机ID不正确。");
        }
        // 厂商ID
        Integer manufacturerId = crossInfoPo.getManufacturerId();
        if (manufacturerId.equals(BasicEnum.ManufacturerEnum.HK.getCode())) {
            Map<String, Object> fieldMap = FieldUtil.fieldMethod(planSendVO);
            // 下发时段方案
            String strResult = ArtemisHttpUtil.doPostStringArtemis(artemisConfig,
                    getPathMapByApiCode("downloadTimeSpands"),
                    JSON.toJSONString(fieldMap), null, null, "application/json", null);
            JSONObject obj = JSON.parseObject(strResult);
            if (obj.getInteger(HK_CODE_KEY).equals(HK_SUCCESS_CODE)) {
                return Result.success();
            }
            return Result.error("下发指令失败");
        } else {
            // todo else
            return null;
        }
    }

    private Map<String, String> getPathMapByApiCode(String apiCode) {
        Map<String, String> res = new HashMap<>();
        HttpRequest httpRequest = new HttpRequest(BasicEnum.ManufacturerEnum.HK.getCode(), apiCode);
        res.put("http://", artemisPath + httpRequest.getUrl());
        return res;
    }

    @Override
    public <T> Result<T> scheduleSend(ScheduleSendVO param) throws Exception {
        CrossInfoPO crossInfoPo = crossInfoMapper.selectByCode(param.getCode());
        if (crossInfoPo == null) {
            throw new ControlException(BODY_NOT_MATCH.getResultCode(), "参数错误，信号机ID不正确。");
        }
        Integer manufacturerId = crossInfoPo.getManufacturerId();
        param.setManufacturerCode(String.valueOf(manufacturerId));
        if (manufacturerId.equals(BasicEnum.ManufacturerEnum.HK.getCode())) {
            return downloadSchedule(param);
        } else {
            // todo else
            return null;
        }
    }

    private <T> Result<T> downloadSchedule(ScheduleSendVO param) throws Exception {
        JSONArray jsonBody = new JSONArray();
        jsonBody.add(param.getCode());
        // 通过获取当前运行的日期数据,获取最新的日期方案id
        String strResult = ArtemisHttpUtil.doPostStringArtemis(artemisConfig,
                getPathMapByApiCode("uploadDate"),
                jsonBody.toString(), null, null, "application/json", null);
        JSONObject obj = JSON.parseObject(strResult);
        //日期 id
        int id = 0;
        if (obj.getInteger(HK_CODE_KEY).equals(HK_SUCCESS_CODE)) {
            // 获取日期列表
            JSONArray schedulesArray = obj.getJSONArray("data").getJSONObject(0).getJSONArray("schedules");
            JSONObject scheduleObj = schedulesArray.getJSONObject(schedulesArray.size() - 1);
            id = scheduleObj.getInteger("id");
        }
        //拼接参数
        JSONArray schedulesArrayParam = new JSONArray();
        // 获取传入的星期数组
        JSONArray weeks = JSON.parseArray(JSON.toJSONString(param.getWeeks()));
        // 获取传入的特殊日期数组
        JSONArray specialDays = JSON.parseArray(JSON.toJSONString(param.getSpecialDays()));
        if (null != weeks) {
            //去重，获取所有的运行方案号
            Set<String> set = new HashSet<>();
            for (int i = 0; i < weeks.size(); i++) {
                String sectionPlanId = weeks.getJSONObject(i).getString("sectionplanid");
                set.add(sectionPlanId);
            }
            if (null != specialDays) {
                for (int i = 0; i < specialDays.size(); i++) {
                    String sectionPlanId = specialDays.getJSONObject(i).getString("sectionplanid");
                    set.add(sectionPlanId);
                }
            }
            int num = 0;
            for (String planNo : set) {
                JSONObject schedulesObjectParam = new JSONObject();
                List<Integer> weeksListParam = new ArrayList<>();
                // 遍历星期列表，拼接星期集合参数
                for (int i = 0; i < weeks.size(); i++) {
                    JSONObject week = weeks.getJSONObject(i);
                    String weekNum = week.getString("weeknum");
                    String sectionplanid = week.getString("sectionplanid");
                    if (planNo.equals(sectionplanid)) {
                        weeksListParam.add(Integer.valueOf(weekNum));
                    }
                }
                // 遍历特殊日期列表，拼接特殊日期集合参数
                if (null != specialDays) {
                    List<String> specialDaysListParam = new ArrayList<>();
                    for (int i = 0; i < specialDays.size(); i++) {
                        JSONObject specialDay = specialDays.getJSONObject(i);
                        String datestr = specialDay.getString("datestr");
                        String sectionplanid = specialDay.getString("sectionplanid");
                        if (planNo.equals(sectionplanid)) {
                            specialDaysListParam.add(datestr);
                        }
                    }
                    schedulesObjectParam.put("dates", specialDaysListParam);
                    //0 说明 dates 内的日期是日期集合，1 说明是dates 内的数据是日期范围
                    schedulesObjectParam.put("isPeriod", 0);
                }
                //scheduleNo 日期号 ，支持返回【1~128】
                if (id + set.size() > 128) {
                    schedulesObjectParam.put("scheduleNo", 128 - set.size() - (++num));
                } else {
                    schedulesObjectParam.put("scheduleNo", id + (++num));
                }
                schedulesObjectParam.put("planNo", planNo);
                schedulesObjectParam.put("weeks", weeksListParam);
                schedulesArrayParam.add(schedulesObjectParam);
            }
        }
        JSONObject bodyObjectParam = new JSONObject();
        bodyObjectParam.put("crossCode", param.getCode());
        bodyObjectParam.put("schedules", schedulesArrayParam);
        // 下发路口运行计划方案
        String updateResult = ArtemisHttpUtil.doPostStringArtemis(artemisConfig,
                getPathMapByApiCode("downloadSchedule"),
                bodyObjectParam.toJSONString(), null, null, "application/json", null);
        JSONObject updateObject = JSON.parseObject(updateResult);
        if (updateObject.getInteger(HK_CODE_KEY).equals(HK_SUCCESS_CODE)) {
            return Result.success();
        }
        return Result.error("下发指令失败");
    }

    @Override
    public <T> Result<T> lockControl(PhaseLockVO phaseLock) throws Exception {
        CrossInfoPO crossInfoPo = crossInfoMapper.selectByCode(phaseLock.getCode());
        if (crossInfoPo == null) {
            throw new ControlException(BODY_NOT_MATCH.getResultCode(), "参数错误，信号机ID不正确。");
        }
        Integer manufacturerId = crossInfoPo.getManufacturerId();
        if (manufacturerId.equals(BasicEnum.ManufacturerEnum.HK.getCode())) {
            if (phaseLock.getCommand().equals(TRUE)) {
                return updateManualLock(phaseLock.getCode(), phaseLock.getLockTime());
            } else {
                return controlSignal(phaseLock.getCode(), UNLOCK_RUNNING_CONTROL);
            }
        } else {
            // todo else
            return null;
        }
    }

    private <T> Result<T> updateManualLock(String code, Integer lockTime) throws Exception {
        Map<String, Object> param = new HashMap<String, Object>() {{
            put("crossCode", code);
            put("controlType", LOCK_RUNNING_CONTROL);
        }};
        // 控制命令为灯态锁定时，设置相关车道参数信息
        // 获取当前运行方案的相位id
        List<SignalRunring> runrings = getSignalInfoVos(code).get(0).getRunrings();
        //HK 为单环
        Integer phaseId = Integer.valueOf(runrings.get(0).getPhaseId());
        //获取某一路口下海康相位和车道关系
        Map<Integer, JSONArray> laneArrayMap = new HashMap<>();
        getPhaseRelationLanes(code, laneArrayMap, new HashMap<>());
        param.put("laneInfos", laneArrayMap.get(phaseId));
        param.put("controlNo", 0);
        //控制时长，单位 s，0 则持续控制
        param.put("duration", lockTime * 60);
        return setSignalControl(param);
    }

    /**
     * 获取某一路口下海康相位和车道关系
     *
     * @param crossCode    信号机唯一编号
     * @param laneArrayMap 相位与车道集合
     * @param laneNoMap    转向和方向与车道关系集合
     */
    private void getPhaseRelationLanes(String crossCode, Map<Integer, JSONArray> laneArrayMap, Map<String, Integer> laneNoMap) throws Exception {
        Map<String, String> path = getPathMapByApiCode("queryPhaseRlatLanes");
        Map<String, String> queryMap = new HashMap<>();
        queryMap.put("crossCode", crossCode);
        String strResult = ArtemisHttpUtil.doPostStringArtemis(artemisConfig, path, null, queryMap, null, "application/json", null);
        JSONObject object = JSON.parseObject(strResult);
        if (HK_SUCCESS_CODE.equals(object.getInteger(HK_CODE_KEY))) {
            JSONObject data = object.getJSONObject("data");
            JSONArray phaseList = data.getJSONArray("phaseList");
            for (int i = 0; i < phaseList.size(); i++) {
                JSONObject lanesObject = phaseList.getJSONObject(i);
                Integer phaseNo = lanesObject.getInteger("phaseNo");
                JSONArray laneList = lanesObject.getJSONArray("lanes");
                // 存入相位与车道集合
                laneArrayMap.put(phaseNo, laneList);
                /*for (int j = 0; j < laneList.size(); j++) {
                    JSONObject laneObject = laneList.getJSONObject(j);
                    String key = laneObject.getString("direction") + SEPARATOR_UNDER_LINE + laneObject.getString("turn");
                    // 存入方向和转向对应的车道编号
                    laneNoMap.put(key, laneObject.getInteger("laneNo"));
                }*/
            }
        }
    }

    @Override
    public <T> Result<T> stepControl(String code, Integer command, Integer stepNum) throws Exception {
        CrossInfoPO crossInfoPo = crossInfoMapper.selectByCode(code);
        if (crossInfoPo == null) {
            throw new ControlException(BODY_NOT_MATCH.getResultCode(), "参数错误，信号机ID不正确。");
        }
        Integer manufacturerId = crossInfoPo.getManufacturerId();
        if (manufacturerId.equals(BasicEnum.ManufacturerEnum.HK.getCode())) {
            if (command.equals(TRUE)) {
                return updateManualStep(code, stepNum);
            } else {
                return controlSignal(code, CANCEL_STEP_CONTROL);
            }
        } else {
            // todo else
            return null;
        }
    }

    @Override
    public <T> Result<T> setSignalControl(String code, Integer command, Integer commandType) throws Exception {
        CrossInfoPO crossInfoPo = crossInfoMapper.selectByCode(code);
        if (crossInfoPo == null) {
            throw new ControlException(BODY_NOT_MATCH.getResultCode(), "参数错误，信号机ID不正确。");
        }
        Integer manufacturerId = crossInfoPo.getManufacturerId();
        if (manufacturerId.equals(BasicEnum.ManufacturerEnum.HK.getCode())) {
            Integer runningMode = command.equals(TRUE) ? commandType : NORMAL_RUNNING_CONTROL;
            return controlSignal(code, runningMode);
        } else {
            // todo else
            return null;
        }
    }

    @Override
    public <T> Result<T> recoverSchedule(String code) throws Exception {
        CrossInfoPO crossInfoPo = crossInfoMapper.selectByCode(code);
        if (crossInfoPo == null) {
            throw new ControlException(BODY_NOT_MATCH.getResultCode(), "参数错误，信号机ID不正确。");
        }
        Integer manufacturerId = crossInfoPo.getManufacturerId();
        if (manufacturerId.equals(BasicEnum.ManufacturerEnum.HK.getCode())) {
            return controlSignal(code, NORMAL_RUNNING_CONTROL);
        } else {
            // todo else
            return null;
        }
    }

    @Override
    public <T> Result<T> phaseTimingSend(String code, List<TimePlanVO> timePlans) throws InterruptedException {
        CrossInfoPO crossInfoPo = crossInfoMapper.selectByCode(code);
        if (crossInfoPo == null) {
            throw new ControlException(BODY_NOT_MATCH.getResultCode(), "参数错误，信号机ID不正确。");
        }
        if (ListUtil.isEmpty(timePlans)) {
            throw new ControlException("参数错误，下发方案信息为空。");
        }
        //开启适配服务
        for (TimePlanVO timePlanVo : timePlans) {
            timePlanVo.setCode(crossInfoPo.getCode());
            timePlanVo.setManufacturerCode(crossInfoPo.getManufacturerId() + "");
        }
        Integer manufacturerId = crossInfoPo.getManufacturerId();
        if (manufacturerId.equals(BasicEnum.ManufacturerEnum.HK.getCode())) {
            for (TimePlanVO timePlanVo : timePlans) {
                if (!phaseTimingSend(timePlanVo)) {
                    //发送失败
                    log.error("批量下发配时方案信息到路口信号机或信号系统中异常！");
                    throw new ControlException("批量下发配时方案信息到路口信号机或信号系统中异常！");
                }
                if (timePlans.size() > 1) {
                    //暂停1秒
                    Thread.sleep(1000);
                }
            }
            return Result.success();
        } else {
            // todo else
            return null;
        }
    }

    private boolean phaseTimingSend(TimePlanVO timePlan) {
        try {
            //获取某一路口下海康相位和车道关系
            Map<Integer, JSONArray> laneArrayMap = new HashMap<>();
            getPhaseRelationLanes(timePlan.getCode(), laneArrayMap, new HashMap<>());
            //拼接参数
            JSONObject bodyObjectParam = new JSONObject();
            JSONArray patternArrayParam = new JSONArray();
            JSONObject patternObjectParam = new JSONObject();
            patternObjectParam.put("patternNo", timePlan.getTimePlanId());
            patternObjectParam.put("offSet", timePlan.getOffset());
            patternObjectParam.put("cycle", timePlan.getCycleLen());
            List<Ring> ringList = timePlan.getRings();
            JSONArray ringsArrayParam = new JSONArray();
            for (Ring ring : ringList) {
                List<Phase> phaseList = ring.getPhases();
                JSONArray phaseArrayParam = new JSONArray();
                for (Phase phase : phaseList) {
                    JSONObject phaseObjectParam = new JSONObject();
                    phaseObjectParam.put("phaseNo", phase.getPhaseId());
                    phaseObjectParam.put("phaseName", phase.getPhaseName());
                    phaseObjectParam.put("phaseLength", phase.getPhaseTime());
                    phaseObjectParam.put("phaseSequence", phase.getPhaseOrderId());
                    //车道列表参数，适配海康车道参数
                    if (laneArrayMap.containsKey(Integer.valueOf(phase.getPhaseId()))) {
                        phaseObjectParam.put("lanes", laneArrayMap.get(Integer.valueOf(phase.getPhaseId())));
                    }
                    phaseArrayParam.add(phaseObjectParam);
                }
                JSONObject ringObjectParam = new JSONObject();
                ringObjectParam.put("ringNo", ring.getRingNo());
                ringObjectParam.put("phaseList", phaseArrayParam);
                ringsArrayParam.add(ringObjectParam);
            }
            patternObjectParam.put("rings", ringsArrayParam);
            patternArrayParam.add(patternObjectParam);
            bodyObjectParam.put("crossCode", timePlan.getCode());
            bodyObjectParam.put("patternList", patternArrayParam);
            //获取下发配时方案的url
            String strResult = ArtemisHttpUtil.doPostStringArtemis(artemisConfig,
                    getPathMapByApiCode("updateTimePlans"),
                    bodyObjectParam.toJSONString(), null, null, "application/json", null);
            JSONObject obj = JSON.parseObject(strResult);
            return HK_SUCCESS_CODE.equals(obj.getInteger(HK_CODE_KEY));
        } catch (Exception e) {
            log.error("系统繁忙，服务器端内部错误！", e);
            throw new ControlException("系统繁忙，服务器端内部错误！");
        }
    }

    @Override
    public <T> Result<T> phaseDiffSend() {
        return null;
    }

    private <T> Result<T> updateManualStep(String signalId, Integer stepNum) {
        JSONObject bodyObjectParam = new JSONObject();
        bodyObjectParam.put("crossCode", signalId);
        bodyObjectParam.put("controlType", STEP_CONTROL);
        int controlNo = 0;
        if (0 != stepNum) {
            //根据当前实时灯态获取运行的相位
            LightsStatusVO lightsStatusVO = getSignalInfoVos(signalId).get(0);
            //暂时按单环处理
            String phaseId = lightsStatusVO.getRunrings().get(0).getPhaseId();
            //获取方案数据列表
            List<CrossPhasePO> baseCrossPhasePlanList = crossPhaseMapper.selectByCrossIdAndPlanId(lightsStatusVO.getCode(), lightsStatusVO.getPhasePlanId());
            Map<String, List<CrossPhasePO>> groupByPhase = baseCrossPhasePlanList.stream().collect(Collectors.groupingBy(CrossPhasePO::getPhaseNo));
            //根据相位获取当前相序
            int newPhaseOrderId = 0;
            for (String key : groupByPhase.keySet()) {
                CrossPhasePO crossPhase = groupByPhase.get(key).get(0);
                if (crossPhase.getPhaseNo().equals(phaseId)) {
                    newPhaseOrderId = (crossPhase.getSort() + stepNum) % groupByPhase.size();
                }
            }
            //根据相序获取相位
            for (String key : groupByPhase.keySet()) {
                CrossPhasePO crossPhase = groupByPhase.get(key).get(0);
                if (newPhaseOrderId == crossPhase.getSort()) {
                    controlNo = Integer.parseInt(crossPhase.getPhaseNo());
                }
            }
        }
        bodyObjectParam.put("controlNo", controlNo);
        return setSignalControl(bodyObjectParam);
    }

    private <T> Result<T> setSignalControl(Map<String, Object> param) {
        try {
            String strResult = ArtemisHttpUtil.doPostStringArtemis(artemisConfig,
                    getPathMapByApiCode("SetSignalControl"),
                    JSON.toJSONString(param), null, null, "application/json", null);
            JSONObject obj = JSON.parseObject(strResult);
            if (HK_SUCCESS_CODE.equals(obj.getInteger(HK_CODE_KEY))) {
                return Result.success();
            }
            return Result.error();
        } catch (Exception e) {
            log.error("系统繁忙，服务器端内部错误！", e);
            throw new ControlException("系统繁忙，服务器端内部错误！");
        }
    }

    /**
     * @param code        信号机编号
     * @param runningMode 0-多时段控制模式 1-关灯、2-黄闪、3-全红、11-取消步进控制、12-灯态锁定、13-灯态解锁
     */
    private <T> Result<T> controlSignal(String code, Integer runningMode) throws Exception {
        //执行恢复时间表方案命令
        if (NORMAL_RUNNING_CONTROL.equals(runningMode) || CANCEL_STEP_CONTROL.equals(runningMode)) {
            //根据实时灯态获取当前运行模式
            List<BaseCrossInfo> baseSignals = new ArrayList<>();
            BaseCrossInfo baseSignal = new BaseCrossInfo();
            baseSignal.setCode(code);
            baseSignal.setManufacturerCode(BasicEnum.ManufacturerEnum.HK.getCode());
            baseSignals.add(baseSignal);
            LightsStatusVO lightsStatusVO = hkLightsStatusService.getHkLightsStatus(baseSignals).get(0);
            //如果当前模式已经是定周期，就不在执行恢复时间表方案指令
            if (FIXED_CYCLE_MODE.equals(Integer.valueOf(lightsStatusVO.getRunMode()))) {
                throw new ControlException("当前控制模式是定周期控制，请勿下发恢复命令");
            }
        }
        Map<String, Object> param = new HashMap<String, Object>() {
            private static final long serialVersionUID = -3468722905882654820L;

            {
                put("crossCode", code);
                put("controlType", runningMode);
                put("controlNo", 0);
                //控制时长，单位 s，0 则持续控制
                put("duration", 0);
            }
        };
        String strResult = ArtemisHttpUtil.doPostStringArtemis(artemisConfig,
                getPathMapByApiCode("SetSignalControl"),
                JSON.toJSONString(param), null, null, "application/json", null);
        JSONObject object = JSON.parseObject(strResult);
        if (HK_SUCCESS_CODE.equals(object.getInteger(HK_CODE_KEY))) {
            return Result.success();
        }
        return Result.error("下发指令失败");
    }

    /**
     * 根据信号机获取灯态列表
     */
    private List<LightsStatusVO> getSignalInfoVos(String code) {
        List<BaseCrossInfo> baseSignals = new ArrayList<>();
        BaseCrossInfo baseSignal = new BaseCrossInfo();
        baseSignal.setCode(code);
        baseSignal.setManufacturerCode(BasicEnum.ManufacturerEnum.HK.getCode());
        baseSignals.add(baseSignal);
        return hkLightsStatusService.getHkLightsStatus(baseSignals);
    }
}
