package net.wanji.utc.service.runninginfo.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hikvision.artemis.sdk.ArtemisHttpUtil;
import com.hikvision.artemis.sdk.config.ArtemisConfig;
import lombok.extern.slf4j.Slf4j;
import net.wanji.utc.common.baseentity.BaseCrossInfo;
import net.wanji.utc.common.commonentity.HttpRequest;
import net.wanji.utc.common.constant.Constants;
import net.wanji.utc.common.typeenum.BasicEnum;
import net.wanji.utc.mapper.CrossInfoMapper;
import net.wanji.utc.mapper.CrossLightsMapper;
import net.wanji.utc.po.CrossLightsPO;
import net.wanji.utc.service.runninginfo.HkLightsStatusService;
import net.wanji.utc.vo.LightsStatusVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Kent HAN
 * @date 2022/11/21 14:07
 */
@Service
@Slf4j
public class HkLightsStatusServiceImpl implements HkLightsStatusService {
    @Value("${signal.manufacturer.hk.artemisPath}")
    private String artemisPath;
    @Autowired
    private ArtemisConfig artemisConfig;
    @Autowired
    private CrossLightsMapper crossLightsMapper;
    @Autowired
    CrossInfoMapper crossInfoMapper;

    @Override
    public List<LightsStatusVO> getHkLightsStatus(List<BaseCrossInfo> baseCrossInfoList) {
        List<LightsStatusVO> result = new ArrayList<>();
        LightsStatusVO infoVo;
        try {
            Map<String, String> path = getPathMapByApiCode("getRealtimeSchemeStateInfo");
            for (BaseCrossInfo entity : baseCrossInfoList) {
                JSONObject jsonBody = new JSONObject();
                jsonBody.put("pageNo", 1);
                jsonBody.put("pageSize", 9999);
                JSONObject searchObj = new JSONObject();
                JSONArray crossCodes = new JSONArray();
                crossCodes.add(entity.getCode());
                searchObj.put("crossCodes", crossCodes);
                jsonBody.put("searchObj", searchObj);
                String strResult = ArtemisHttpUtil.doPostStringArtemis(artemisConfig, path, jsonBody.toJSONString(),
                        null, null, "application/json", null);
                JSONObject object = JSON.parseObject(strResult);

                String crossId = crossInfoMapper.selectIdByCode(entity.getCode());
                List<CrossLightsPO> crossLightsPOList = crossLightsMapper.selectByCrossId(crossId);

                if(Constants.HK_SUCCESS_CODE.equals(object.getInteger(Constants.HK_CODE_KEY))) {
                    JSONObject data = object.getJSONObject("data");
                    JSONArray list = data.getJSONArray("list");
                    for (int i = 0; i < list.size(); i++) {
                        infoVo = new LightsStatusVO();
                        JSONObject jsonObject = list.getJSONObject(i);
                        String patternNo = jsonObject.getString("patternNo");               // 方案编号
                        JSONArray rings = jsonObject.getJSONArray("rings");                 // 环列表
                        JSONArray channelState = jsonObject.getJSONArray("channelState");   // 灯组列表
                        infoVo.setRunMode(jsonObject.getString("controlType"));             // 控制模式
                        infoVo.setPhasePlanId(patternNo);                                       // 相位方案号
                        int cycleLen = jsonObject.getIntValue("cycle");                     // 周期时长
                        int curRunTime = jsonObject.getIntValue("curRunTime");              // 当前运行时长
                        int cycleCountDown = cycleLen - curRunTime;                             // 周期剩余时长
                        infoVo.setCode(entity.getCode());
                        infoVo.setManufacturerCode(entity.getManufacturerCode());
                        infoVo.setCycleCountDown(cycleCountDown);
                        infoVo.setCycleLen(cycleLen);
                        // 循环相位列表
                        Map<String,Object> phaseMap = new HashMap<>();
                        for (int j = 0; j < rings.size(); j++) {
                            JSONObject jsonObj = rings.getJSONObject(j);
                            JSONArray phaseList = jsonObj.getJSONArray("phaseList");
                            for (int k = 0; k < phaseList.size(); k++) {
                                JSONObject phase = phaseList.getJSONObject(k);
                                String phaseNo = phase.getString("phaseNo");
                                Integer vehicleStatus = phase.getInteger("vehicleStatus");
                                Integer phaseLength = phase.getInteger("phaseLength");
                                Integer red = phase.getInteger("red");
                                Integer yellow = phase.getInteger("yellow");
                                Integer green = phaseLength - yellow - red;
                                phaseMap.put(phaseNo, green);
                                if(2 != vehicleStatus){ // 2红灯
                                    infoVo.setPhaseId(phaseNo);
                                }
                            }
                        }
                        infoVo.setPhaseMap(phaseMap);

                        Map<String,Object> dirLampGroupMap = new HashMap<>();
                        // 循环构建灯组对象（从灯组表获取灯组信息）
                        for (int k = 0; k < channelState.size(); k++) {
                            JSONObject channelObj = channelState.getJSONObject(k);
                            int channelNo = channelObj.getIntValue("channelNo");
                            String color = transferHKLampGroup(channelObj.getIntValue("state"));
                            for (CrossLightsPO po : crossLightsPOList){
                                if(Integer.parseInt(po.getLightsNo()) == channelNo) {
                                    String direction = po.getDir().toString();
                                    String turn = po.getTurn().toString();
                                    if(dirLampGroupMap.get(direction) != null){
                                        Map<String,String> mstr = (Map<String,String>)dirLampGroupMap.get(direction);
                                        mstr.put(turn,color);
                                        dirLampGroupMap.put(direction,mstr);
                                    } else {
                                        Map<String,String> lamp = new HashMap<>();
                                        lamp.put(turn,color);
                                        dirLampGroupMap.put(direction,lamp);
                                    }
                                }
                            }
                        }
                        infoVo.setDirLampGroupMap(dirLampGroupMap);
                        result.add(infoVo);
                    }
                }
            }
        }  catch(Exception e){
            log.error("系统繁忙，服务器端内部错误！", e);
        }
        return result;
    }


    /**
     *  灯组颜色转换
     * @param state
     * @return
     */
    private String transferHKLampGroup(int state){
        String color = "";
        switch (state) {
            case 1: color = "green"; break;
            case 2: color = "red"; break;
            case 3: color = "yellow"; break;
            case 4: color = "greenFlash"; break;
            case 5: color = "yellowFlash"; break;
            case 6: color = "red"; break;
            case 7: color = "lightsOut"; break;
            case 8: color = "redFlash"; break;
            case 9: color = "redYellow"; break;
            default: color = ""; break;
        }
        return color;
    }

    private Map<String, String> getPathMapByApiCode(String apiCode) {
        Map<String, String> res = new HashMap<>();
        HttpRequest httpRequest = new HttpRequest(BasicEnum.ManufacturerEnum.HK.getCode(), apiCode);
        res.put("http://", artemisPath + httpRequest.getUrl());
        return res;
    }
}
