package net.wanji.utc.service.runninginfo.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hikvision.artemis.sdk.ArtemisHttpUtil;
import com.hikvision.artemis.sdk.config.ArtemisConfig;
import lombok.extern.slf4j.Slf4j;
import net.wanji.utc.common.baseentity.BaseCrossInfo;
import net.wanji.utc.common.commonentity.HttpRequest;
import net.wanji.utc.common.constant.Constants;
import net.wanji.utc.common.constant.RedisKeyConst;
import net.wanji.utc.common.typeenum.BasicEnum;
import net.wanji.utc.common.typeenum.HkControlModeEnum;
import net.wanji.utc.po.SignalStatusLogPO;
import net.wanji.utc.service.runninginfo.HkRunningStatusService;
import net.wanji.utc.util.RedisUtil;
import net.wanji.utc.vo.LightsStatusVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Kent HAN
 * @date 2022/11/21 10:05
 */
@Slf4j
@Service
public class HkRunningStatusServiceImpl implements HkRunningStatusService {
    @Value("${signal.manufacturer.hk.artemisPath}")
    private String artemisPath;
    @Autowired
    private ArtemisConfig artemisConfig;
    @Autowired
    private RedisUtil redisUtil;

    @Override
    public List<SignalStatusLogPO> getHkRunningStatus(List<BaseCrossInfo> baseCrossInfoList) {
        List<SignalStatusLogPO> result = new ArrayList<>();
        SignalStatusLogPO po;
        try {
            Map<String, String> runningPath = getPathMapByApiCode("getWorkingStateBatch");
            for (BaseCrossInfo baseCrossInfo : baseCrossInfoList) {
                JSONArray jsonBody = new JSONArray();
                jsonBody.add(baseCrossInfo.getCode());
                String strResult = ArtemisHttpUtil.doPostStringArtemis(artemisConfig, runningPath, jsonBody.toJSONString(),
                        null, null, "application/json", null);
                JSONObject object = JSON.parseObject(strResult);
                if (Constants.HK_SUCCESS_CODE.equals(object.getInteger(Constants.HK_CODE_KEY))) {
                    JSONArray data = object.getJSONArray("data");
                    for (int i = 0; i < data.size(); i++) {
                        JSONObject obj =  data.getJSONObject(i);
                        po = new SignalStatusLogPO();
                        po.setSignalId(baseCrossInfo.getCode());
                        po.setStatus(transferStatus(obj.getInteger("state")));
                        // 告警信息
                        Integer hasFault = obj.getInteger("hasFault");
                        if (hasFault == 0) {
                            po.setFaultType(hasFault);
                        }
                        // 从Redis中查询控制类型
                        String lightsStatusVOStr = redisUtil.getHash(RedisKeyConst.KEY_PREFIX + RedisKeyConst.LIGHTS_STATUS,
                                baseCrossInfo.getManufacturerCode() + Constants.SEPARATOR_UNDER_LINE + po.getSignalId());
                        LightsStatusVO lightsStatusVO = JSON.parseObject(lightsStatusVOStr, LightsStatusVO.class);
                        String runMode = lightsStatusVO.getRunMode();
                        int runModeInt = Integer.parseInt(runMode);
                        Integer controlType = HkControlModeEnum.getCodeByHkCode(runModeInt);
                        po.setControlType(controlType);
                        result.add(po);
                    }
                }
            }
        } catch(Exception e){
            log.error("系统繁忙，服务器端内部错误！", e);
        }
        return result;
    }

    private Map<String, String> getPathMapByApiCode(String apiCode) {
        Map<String, String> res = new HashMap<>();
        HttpRequest httpRequest = new HttpRequest(BasicEnum.ManufacturerEnum.HK.getCode(), apiCode);
        res.put("http://", artemisPath + httpRequest.getUrl());
        return res;
    }

    /**
     * 转换海康信号机运行状态，0 离线 1 在线
     * @param state  海康信号机状态定义 0 在线 1 离线
     * @return
     */
    private int transferStatus(int state) {
        return 0 == state ? 1 : 0;
    }
}
