package net.wanji.utc.service.runninginfo.impl;

import com.alibaba.fastjson.JSON;
import net.wanji.utc.common.baseentity.BaseCrossInfo;
import net.wanji.utc.common.constant.Constants;
import net.wanji.utc.common.constant.RedisKeyConst;
import net.wanji.utc.common.typeenum.BasicEnum;
import net.wanji.utc.mapper.CrossInfoMapper;
import net.wanji.utc.mapper.ManufacturerInfoMapper;
import net.wanji.utc.mapper.SignalStatusLogMapper;
import net.wanji.utc.po.CrossInfoPO;
import net.wanji.utc.po.ManufacturerInfoPO;
import net.wanji.utc.po.SignalStatusLogPO;
import net.wanji.utc.service.runninginfo.HkLightsStatusService;
import net.wanji.utc.service.runninginfo.HkRunningStatusService;
import net.wanji.utc.service.runninginfo.SignalStatusService;
import net.wanji.utc.util.ListUtil;
import net.wanji.utc.util.RedisUtil;
import net.wanji.utc.vo.LightsStatusVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @date 2022/11/21 9:11
 */
@Service
public class SignalStatusServiceImpl implements SignalStatusService {
    @Autowired
    ManufacturerInfoMapper manufacturerInfoMapper;
    @Autowired
    CrossInfoMapper crossInfoMapper;
    @Autowired
    HkRunningStatusService hkRunningStatusService;
    @Autowired
    HkLightsStatusService hkLightsStatusService;
    @Autowired
    RedisUtil redisUtil;
    @Autowired
    SignalStatusLogMapper signalStatusLogMapper;

    @Override
    public void runningStatusAlarm() {
        BasicEnum.ManufacturerEnum[] manufacturerEnums = BasicEnum.ManufacturerEnum.values();
        for (BasicEnum.ManufacturerEnum manufacturerEnum : manufacturerEnums) {
            List<BaseCrossInfo> baseCrossInfoList = getBaseCrossInfoList(manufacturerEnum);
            if (baseCrossInfoList == null) continue;
            List<SignalStatusLogPO> signalStatusLogPOList = null;
            if(manufacturerEnum.getCode().equals(BasicEnum.ManufacturerEnum.HK.getCode())) {
                // 海康
                // 3.2.5信号机运行状态和告警信息
                signalStatusLogPOList = hkRunningStatusService.getHkRunningStatus(baseCrossInfoList);
            } else {
                // todo 其他厂商
//                ParamEntity<List<BaseSignal>> param = new ParamEntity<>();
//                param.setDatacontent(baseSignals);
//                param.setSourcetype(vendorTypeEnum.getNick());
//                param.setSystemtype(Const.SYSTEM_TYPE);
//                param.setUpdatetime(new Date());
//                statusVos = signalRunService.queryRunState(param).getDatacontent();
            }
            // 更新Redis
            if (null != signalStatusLogPOList) {
                for (SignalStatusLogPO signalStatusLogPO : signalStatusLogPOList) {
                    String field = manufacturerEnum.getCode() + Constants.SEPARATOR_UNDER_LINE +
                            signalStatusLogPO.getSignalId();
                    redisUtil.setHash(RedisKeyConst.KEY_PREFIX + RedisKeyConst.TELESEME_STATUS, field,
                            signalStatusLogPO.getStatus()+"");
                }
                // 更新日志数据库
                if (ListUtil.isNotEmpty(signalStatusLogPOList)) {
                    signalStatusLogMapper.insertBatch(signalStatusLogPOList);
                }
            }
        }
    }

    @Override
    public void lightStatus() {
        BasicEnum.ManufacturerEnum[] manufacturerEnums = BasicEnum.ManufacturerEnum.values();
        for (BasicEnum.ManufacturerEnum manufacturerEnum : manufacturerEnums) {
            List<BaseCrossInfo> baseCrossInfoList = getBaseCrossInfoList(manufacturerEnum);
            if (baseCrossInfoList == null) continue;
            List<LightsStatusVO> lightsStatusVOList = null;
            if(manufacturerEnum.getCode().equals(BasicEnum.ManufacturerEnum.HK.getCode())) {
                // 海康
                // 3.2.4灯态
                lightsStatusVOList = hkLightsStatusService.getHkLightsStatus(baseCrossInfoList);
            } else {
                // todo 其他厂商
            }
            // 更新Redis
            if (lightsStatusVOList != null) {
                for (LightsStatusVO lightsStatusVO : lightsStatusVOList) {
                    String field = manufacturerEnum.getCode() + Constants.SEPARATOR_UNDER_LINE +
                            lightsStatusVO.getCode();
                    redisUtil.setHash(RedisKeyConst.KEY_PREFIX + RedisKeyConst.LIGHTS_STATUS, field,
                            JSON.toJSONString(lightsStatusVO));
                }
            }
        }
    }

    private List<BaseCrossInfo> getBaseCrossInfoList(BasicEnum.ManufacturerEnum manufacturerEnum) {
        // 查询路口信息
        ManufacturerInfoPO manufacturerInfoPO = manufacturerInfoMapper.selectByCode(manufacturerEnum.getCode());
        Integer manufacturerId = manufacturerInfoPO.getId();
        List<CrossInfoPO> crossInfoPOList = crossInfoMapper.selectByManufacturerId(manufacturerId);
        if (ListUtil.isEmpty(crossInfoPOList)) {
            return null;
        }
        // 信号机接口入参
        List<BaseCrossInfo> baseCrossInfoList = new ArrayList<>();
        for (CrossInfoPO infoPO : crossInfoPOList) {
            BaseCrossInfo baseCrossInfo = new BaseCrossInfo();
            baseCrossInfo.setCode(infoPO.getCode());
            baseCrossInfo.setManufacturerCode(manufacturerEnum.getCode());
            baseCrossInfoList.add(baseCrossInfo);
        }
        return baseCrossInfoList;
    }
}
