package net.wanji.utc.service.staticinfo.impl;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hikvision.artemis.sdk.ArtemisHttpUtil;
import com.hikvision.artemis.sdk.config.ArtemisConfig;
import net.wanji.utc.common.commonentity.HttpRequest;
import net.wanji.utc.common.constant.Constants;
import net.wanji.utc.common.typeenum.BasicEnum;
import net.wanji.utc.mapper.CrossInfoMapper;
import net.wanji.utc.mapper.CrossPlanMapper;
import net.wanji.utc.mapper.CrossSchedulesMapper;
import net.wanji.utc.po.CrossSchedulesPO;
import net.wanji.utc.service.staticinfo.HkCrossSchedulesService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author Kent HAN
 * @date 2022/11/18 17:20
 */
@Service
public class HkCrossSchedulesServiceImpl implements HkCrossSchedulesService {
    @Value("${signal.manufacturer.hk.artemisPath}")
    private String artemisPath;
    @Autowired
    private ArtemisConfig artemisConfig;
    @Autowired
    CrossInfoMapper crossInfoMapper;
    @Autowired
    CrossPlanMapper crossPlanMapper;
    @Autowired
    CrossSchedulesMapper crossSchedulesMapper;

    @Override
    public void crossSchedules(List<String> crossIdList) throws Exception {
        // 请求3.1.9接口，获取时间表
        Map<String, String> path = getPathMapByApiCode("uploadDate");
        List<String> crossCodeList = crossInfoMapper.selectCrossCodesByIds(crossIdList);
        String body = JSONArray.toJSONString(crossCodeList);
        String responseStr = ArtemisHttpUtil.doPostStringArtemis(artemisConfig, path, body, null,
                null, "application/json", null);
        JSONObject responseObj = JSON.parseObject(responseStr);
        if(Constants.HK_SUCCESS_CODE.equals(responseObj.getInteger(Constants.HK_CODE_KEY))) {
            JSONArray data = responseObj.getJSONArray("data");
            for (int i = 0; i < data.size(); i++) {
                ArrayList<CrossSchedulesPO> crossSchedulesPOList = new ArrayList<>();
                JSONObject dataEle = data.getJSONObject(i);
                JSONArray schedules = dataEle.getJSONArray("schedules");
                String crossCode = dataEle.getString("crossCode");
                String crossId = crossInfoMapper.selectIdByCode(crossCode);
                for (int j = 0; j < schedules.size(); j++) {
                    JSONObject schedule = schedules.getJSONObject(j);
                    JSONArray weeks = schedule.getJSONArray("weeks");
                    String desc = schedule.getString("desc");
                    Integer planNo = schedule.getInteger("planNo");
                    Integer planId = crossPlanMapper.selectIdByNo(crossId, planNo);
                    for (int k = 0; k < weeks.size(); k++) {
                        Integer week = weeks.getInteger(k);
                        CrossSchedulesPO crossSchedulesPO = new CrossSchedulesPO();
                        crossSchedulesPO.setName(desc);
                        crossSchedulesPO.setCrossId(crossId);
                        crossSchedulesPO.setPlanId(planId);
                        crossSchedulesPO.setWeek(week);
                        if (week != 0) {
                            crossSchedulesPOList.add(crossSchedulesPO);
                        } else {
                            // 特殊日期
                            JSONArray dates = schedule.getJSONArray("dates");
                            for (int m = 0; m < dates.size(); m++) {
                                JSONObject date = dates.getJSONObject(m);
                                JSONArray days = date.getJSONArray("days");
                                for (int n = 0; n < days.size(); n++) {
                                    Integer day = days.getInteger(n);
                                    Date specialDate = toDate(date, day);
                                    crossSchedulesPO.setSpecialDate(specialDate);
                                    crossSchedulesPOList.add(crossSchedulesPO);
                                }
                            }
                        }
                    }
                }
                // 插入数据库
                crossSchedulesMapper.deleteByCrossId(crossId);
                crossSchedulesMapper.insertBatch(crossSchedulesPOList);
            }
        }

    }

    private Date toDate(JSONObject date, Integer dayInt) {
        String year = date.getString("year");
        String month = date.getString("month");
        String day = dayInt.toString();
        if (month.length() == 1) {
            month = "0" + month;
        }
        if (day.length() == 1) {
            day = "0" + day;
        }
        String dateStr = year + "-" + month + "-" + day;
        Date parse = DateUtil.parse(dateStr);
        return parse;
    }

    private Map<String, String> getPathMapByApiCode(String apiCode) {
        Map<String, String> res = new HashMap<>();
        HttpRequest httpRequest = new HttpRequest(BasicEnum.ManufacturerEnum.HK.getCode(), apiCode);
        res.put("http://", artemisPath + httpRequest.getUrl());
        return res;
    }
}
