package net.wanji.utc.service.staticinfo.impl;

import lombok.RequiredArgsConstructor;
import net.wanji.utc.common.genericentity.ManufacturerRes;
import net.wanji.utc.common.typeenum.BasicEnum;
import net.wanji.utc.mapper.CrossInfoMapper;
import net.wanji.utc.mapper.ManufacturerInfoMapper;
import net.wanji.utc.po.CrossInfoPO;
import net.wanji.utc.po.ManufacturerInfoPO;
import net.wanji.utc.common.baseentity.BaseCrossInfo;
import net.wanji.utc.common.genericentity.OutVO;
import net.wanji.utc.service.staticinfo.*;
import net.wanji.utc.util.ListUtil;
import net.wanji.utc.vo.CrossSchedulesInVO;
import net.wanji.utc.vo.DetailCrossInfoVO;
import net.wanji.utc.vo.PlanSectionInVO;
import net.wanji.utc.vo.SchemePhaseLightsInVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @author Kent HAN
 * @date 2022/11/15 10:32
 */
@Service
@RequiredArgsConstructor
public class StaticInfoServiceImpl implements StaticInfoService {
    private final HkCrossInfoService hkCrossInfoService;
    @Autowired
    CrossInfoMapper crossInfoMapper;
    @Autowired
    OthersStaticInfoService othersStaticInfoService;
    @Autowired
    ManufacturerInfoMapper manufacturerInfoMapper;
    @Autowired
    HkSchemePhaseLightsService hkSchemePhaseLightsService;
    @Autowired
    HkPlanSectionService hkPlanSectionService;
    @Autowired
    HkCrossSchedulesService hkCrossSchedulesService;

    @Override
    public ManufacturerRes<DetailCrossInfoVO> crossBasicInfo(OutVO<BaseCrossInfo> outVO) {
        List<CrossInfoPO> crossInfoPOList = new ArrayList<>();
        ManufacturerRes<DetailCrossInfoVO> res = new ManufacturerRes<>();
        if (Objects.equals(BasicEnum.ManufacturerEnum.HK.getCode(), outVO.getManufacturerCode())) {
            crossInfoPOList = hkCrossInfoService.hkCrossBasicInfo();
        } else {
            // todo 其他厂商
            res = othersStaticInfoService.crossBasicInfo(outVO.getManufacturerCode());
            if (null == res) {
                return null;
            }
            List<DetailCrossInfoVO> dataContent = res.getDatalist();
            if (ListUtil.isEmpty(dataContent)) {
                return null;
            }
            for (DetailCrossInfoVO vo : dataContent) {
                String id = vo.getCode();
                CrossInfoPO crossInfoPO = crossInfoMapper.selectByPrimaryKey(id);
                if (null != crossInfoPO) {
                    continue;
                }
                crossInfoPO = new CrossInfoPO();
                // 用信号机ID作为路口ID
                crossInfoPO.setId(id);
                crossInfoPO.setName(vo.getCrossName());
                crossInfoPO.setCode(id);
                ManufacturerInfoPO manufacturerInfoPO = manufacturerInfoMapper.selectByCode(vo.getManufacturerCode());
                if (manufacturerInfoPO != null) {
                    crossInfoPO.setManufacturerId(manufacturerInfoPO.getId());
                }
                crossInfoPO.setIp(vo.getIp());
                crossInfoPO.setPort(vo.getPort());
                // 经纬度
                Double longitude = vo.getLon();
                Double latitude = vo.getLat();
                crossInfoPO.setLocation(longitude + "," + latitude);
                crossInfoPO.setVersion(vo.getVersion());
                crossInfoPO.setModel(vo.getModel());
                crossInfoPO.setInstallTime(vo.getInstallTime());

                crossInfoPOList.add(crossInfoPO);
            }
        }
        //存储信号机信息
        if (crossInfoPOList.size() == 0) {
            return null;
        }
        crossInfoMapper.insertBatch(crossInfoPOList);
        return res;
    }

    @Override
    public void schemePhaseLights(SchemePhaseLightsInVO schemePhaseLightsInVO) throws Exception {
        String crossId = schemePhaseLightsInVO.getCrossId();
        // 获取厂商ID和信号机ID
        CrossInfoPO crossInfoPO = crossInfoMapper.selectByPrimaryKey(crossId);
        String crossCode = crossInfoPO.getCode();
        Integer manufacturerId = crossInfoPO.getManufacturerId();
        // 获取厂商缩写
        ManufacturerInfoPO manufacturerInfoPO = manufacturerInfoMapper.selectById(manufacturerId);
        String manufacturerCode = manufacturerInfoPO.getCode();
        if (Objects.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerCode)) {
            // 海康
            hkSchemePhaseLightsService.hkSchemePhaseLights(crossId, crossCode);
        } else {
            // todo 其他厂商
        }
    }

    @Override
    public void planSection(PlanSectionInVO planSectionInVO) throws Exception {
        String crossId = planSectionInVO.getCrossId();
        // 获取厂商ID和信号机ID
        CrossInfoPO crossInfoPO = crossInfoMapper.selectByPrimaryKey(crossId);
        String crossCode = crossInfoPO.getCode();
        Integer manufacturerId = crossInfoPO.getManufacturerId();
        // 获取厂商缩写
        ManufacturerInfoPO manufacturerInfoPO = manufacturerInfoMapper.selectById(manufacturerId);
        String manufacturerCode = manufacturerInfoPO.getCode();
        Integer planNo = planSectionInVO.getPlanNo();
        if (Objects.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerCode)) {
            // 海康
            hkPlanSectionService.planSection(crossCode, planNo, crossId);
        } else {
            // todo 其他厂商
        }
    }

    @Override
    public void crossSchedules(CrossSchedulesInVO crossSchedulesInVO) throws Exception {
        String manufacturerCode = crossSchedulesInVO.getManufacturerCode();
        List<String> crossIdList = crossSchedulesInVO.getCrossIdList();
        if (Objects.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerCode)) {
            // 海康
            hkCrossSchedulesService.crossSchedules(crossIdList);
        } else {
            // todo 其他厂商
        }

    }
}
