package net.wanji.utc.service.systemadmin.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import net.wanji.utc.common.exception.IpPortException;
import net.wanji.utc.common.exception.PortFormatException;
import net.wanji.utc.mapper.CrossInfoMapper;
import net.wanji.utc.mapper.ManufacturerInfoMapper;
import net.wanji.utc.po.CrossInfoPO;
import net.wanji.utc.po.ManufacturerInfoPO;
import net.wanji.utc.service.systemadmin.CrossInfoService;
import net.wanji.utc.util.PageUtils;
import net.wanji.utc.vo.systemadmin.CrossInfoInsertOrUpdateInVO;
import net.wanji.utc.vo.systemadmin.CrossInfoListInVO;
import net.wanji.utc.vo.systemadmin.CrossInfoListOutVO;
import net.wanji.utc.vo.systemadmin.DeleteByStringIdListInVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * @author Kent HAN
 * @date 2022/11/25 8:54
 */
@Service
public class CrossInfoServiceImpl implements CrossInfoService {
    @Autowired
    ManufacturerInfoMapper manufacturerInfoMapper;
    @Autowired
    CrossInfoMapper crossInfoMapper;

    @Override
    public PageInfo<CrossInfoListOutVO> list(CrossInfoListInVO crossInfoListInVO) {
        Integer pageNum = crossInfoListInVO.getPageNum();
        Integer pageSize = crossInfoListInVO.getPageSize();

        String crossName = crossInfoListInVO.getCrossName();

        String manufacturerNick = crossInfoListInVO.getManufacturerNick();
        Integer manufacturerId = null;
        if (manufacturerNick != null) {
            manufacturerId = manufacturerInfoMapper.selectIdByNick(manufacturerNick);
        }

        PageHelper.startPage(pageNum, pageSize);
        List<CrossInfoPO> crossInfoPOList = crossInfoMapper.selectByOptionals(crossName, manufacturerId);
        PageInfo<CrossInfoPO> crossInfoPOPageInfo = new PageInfo<>(crossInfoPOList);
        // 转换为VO类型的PageInfo对象
        PageInfo<CrossInfoListOutVO> crossInfoListOutVOPageInfo = PageUtils.PageInfo2PageInfoVo(crossInfoPOPageInfo);
        for (CrossInfoPO crossInfoPO : crossInfoPOList) {
            CrossInfoListOutVO crossInfoListOutVO = new CrossInfoListOutVO();
            BeanUtils.copyProperties(crossInfoPO, crossInfoListOutVO);
            Integer manufacturerIdInPage = crossInfoPO.getManufacturerId();
            ManufacturerInfoPO manufacturerInfoPO = manufacturerInfoMapper.selectById(manufacturerIdInPage);
            String nickName = manufacturerInfoPO.getNickName();
            crossInfoListOutVO.setManufacturerNick(nickName);
            crossInfoListOutVOPageInfo.getList().add(crossInfoListOutVO);
        }

        return crossInfoListOutVOPageInfo;
    }

    @Override
    public void insertOrUpdate(CrossInfoInsertOrUpdateInVO inVO) {
        CrossInfoPO crossInfoPO = new CrossInfoPO();
        BeanUtils.copyProperties(inVO, crossInfoPO);

        // IP和端口
        String ipPort = inVO.getIpPort();
        String[] split = ipPort.split(":");
        if (split.length != 2) {
            throw new IpPortException("IP/端口格式错误，正确格式为 192.168.1.23:8080");
        }
        String portStr = split[1];
        Integer port = null;
        try {
            port = Integer.parseInt(portStr);
        } catch (NumberFormatException e){
            throw new PortFormatException("端口必须为数字");
        }
        crossInfoPO.setIp(split[0]);
        crossInfoPO.setPort(port);

        // 厂商ID
        String manufacturerNick = inVO.getManufacturerNick();
        Integer manufacturerId = manufacturerInfoMapper.selectIdByNick(manufacturerNick);
        crossInfoPO.setManufacturerId(manufacturerId);

        // 安装时间
        String installTimeStr = inVO.getInstallTime();
        Date installTime = DateUtil.parse(installTimeStr);
        crossInfoPO.setInstallTime(installTime);

        String id = inVO.getId();
        if (id == null || "".equals(id)) {
            // 添加
            crossInfoPO.setId(IdUtil.simpleUUID());
            crossInfoMapper.insertOne(crossInfoPO);
        } else {
            // 修改
            crossInfoMapper.updateOne(crossInfoPO);
        }
    }

    @Override
    public void delete(DeleteByStringIdListInVO inVO) {
        List<String> ids = inVO.getIds();
        crossInfoMapper.deleteBatch(ids);
    }
}
