package net.wanji.utc.vo.signal;

import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import net.wanji.utc.common.baseentity.BaseCrossInfo;

import java.util.List;
import java.util.Map;

/**
 * 信号灯态基础信息实体
 */
@Data
public class SignalLightStateVo extends BaseCrossInfo {

    /**
     * 运行模式
     */
    @ApiModelProperty(value = "运行模式 `2=手动锁定当前相位`，`3=手动全红`，`4=手动黄闪`，`5=手动关灯`，" +
            "`6=全红闪控制`，`7=全黄控制`，`8=正常按计划运行`，`9=手动步进控制`，`10=自动步进控制`，`11=恢复时间表`，`255=临时控制方案`")
    private String runMode;

    /**
     * 控制模式
     */
    @ApiModelProperty(value = "控制模式")
    private String controlMode;

    /**
     * 相位方案号
     */
    @ApiModelProperty(value = "相位方案号")
    private String phasePlanId;

    /**
     * 相位配时方案
     */
    @ApiModelProperty(value = "相位配时方案")
    private String timePlanId;

    /**
     * 方案开始时间
     */
    @ApiModelProperty(value = "方案开始时间")
    private String planStartTime;

    /**
     * 方案运行时间
     */
    @ApiModelProperty(value = "方案运行时间")
    private Integer runTime;

    /**
     * 相位周期剩余时长
     */
    @ApiModelProperty(value = "相位周期剩余时长")
    private Integer cycleCountDown;

    /**
     * 相位周期时长
     */
    @ApiModelProperty(value = "相位周期时长")
    private Integer cycleLen;

    /**
     * 运行环数组
     */
    private List<SignalRingVo> rings;

    /**
     * 当前相位号
     */
    @ApiModelProperty(value = "当前相位号")
    private String phaseId;

    /**
     * 当前方案号
     */
    @ApiModelProperty(value = "当前方案号")
    private String planId;

    /**
     * 灯组状态
     */
    @ApiModelProperty(value = "灯组状态")
    private Map<String, Object> dirLampGroupMap;

    /**
     * 相位对象：key：相位编号，value：绿灯时长
     */
    @ApiModelProperty(value = "相位对象：key相位编号，value绿灯时长")
    private Map<String, Object> phaseMap;

    public void setCycleCountDown(Integer cycleCountDown) {
        if (null == cycleCountDown) {
            this.cycleCountDown = -1;
        } else {
            this.cycleCountDown = cycleCountDown;
        }
    }

    public Integer getCycleCountDown() {
        if (null == cycleCountDown) {
            return -1;
        }
        return cycleCountDown;
    }
}
